// -*- C++ -*-
/*!
 * @file  ev3allsensor.cpp
 * @brief ev3 use all sensor
 * @date $Date$
 *
 * $Id$
 */

#include "ev3allsensor.h"

// Module specification
// <rtc-template block="module_spec">
static const char* ev3allsensor_spec[] =
  {
    "implementation_id", "ev3allsensor",
    "type_name",         "ev3allsensor",
    "description",       "ev3 use all sensor",
    "version",           "1.0.0",
    "vendor",            "fsk",
    "category",          "ev",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.back_speed", "0.01",
    "conf.default.front_speed", "0.01",
    "conf.default.turn_speed", "0.01",
    "conf.default.turn_time", "1",
    "conf.default.back_time", "1",
    "conf.default.ultrasonicRange", "8",

    // Widget
    "conf.__widget__.back_speed", "text",
    "conf.__widget__.front_speed", "text",
    "conf.__widget__.turn_speed", "text",
    "conf.__widget__.turn_time", "text",
    "conf.__widget__.back_time", "text",
    "conf.__widget__.ultrasonicRange", "text",
    // Constraints

    "conf.__type__.back_speed", "float",
    "conf.__type__.front_speed", "float",
    "conf.__type__.turn_speed", "float",
    "conf.__type__.turn_time", "int",
    "conf.__type__.back_time", "int",
    "conf.__type__.ultrasonicRange", "float",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ev3allsensor::ev3allsensor(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_ultrasonicIn("ultrasonic", m_ultrasonic),
    m_colorIn("color", m_color),
    m_touchIn("touch", m_touch),
    m_velOut("vel", m_vel)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ev3allsensor::~ev3allsensor()
{
}



RTC::ReturnCode_t ev3allsensor::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("ultrasonic", m_ultrasonicIn);
  addInPort("color", m_colorIn);
  addInPort("touch", m_touchIn);
  
  // Set OutPort buffer
  addOutPort("vel", m_velOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("back_speed", m_back_vel, "0.01");
  bindParameter("front_speed", m_front_vel, "0.01");
  bindParameter("turn_speed", m_turn_vel, "0.01");
  bindParameter("turn_time", m_turn_time, "1");
  bindParameter("back_time", m_back_time, "1");
  bindParameter("ultrasonicRange", m_ultrasonicRange, "8");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ev3allsensor::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3allsensor::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3allsensor::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ev3allsensor::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "onActive" << std::endl;
	toward = 0;
	changetoward = true;
	m_vel.data.vx = m_front_vel;
	m_vel.data.vy = 0;
	m_vel.data.va = 0;
	f_v = m_front_vel;
	b_t = m_back_time;
	b_v = m_back_vel;
	t_v = m_turn_vel;
	t_t = m_turn_time;
	u_v = m_ultrasonicRange;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ev3allsensor::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "onDeactive" << std::endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ev3allsensor::onExecute(RTC::UniqueId ec_id)
{
	//コンフィギレーションの値変更時に表示
	if (f_v != m_front_vel || b_v != m_back_vel || b_t != m_back_time ||
		t_v != m_turn_vel || t_t != m_turn_time || u_v != m_ultrasonicRange){
		std::cout << "front vel:" << m_front_vel << " back vel:" << m_back_vel << " back time:" << m_back_time;
		std::cout << " turn vel:" << m_turn_vel << " turn time:" << m_turn_time << " ultrasonic:" << m_ultrasonicRange << std::endl;

		m_vel.data.vx = m_front_vel;
		f_v = m_front_vel;
		b_v = m_back_vel;
		b_t = m_back_time;
		t_v = m_turn_vel;
		t_t = m_turn_time;
		u_v = m_ultrasonicRange;
	}
	//カラーセンサーの値読み込み
	if (m_colorIn.isNew())
	{
		m_colorIn.read();
		std::string color_data = (const char*)m_color.data;
		
		if (changetoward == true){
			if (color_data == "2")toward = 2;
			else if (color_data == "3")toward = 1;
			else toward = 0;
			if (toward != 0 && changetoward == true){
				changetoward = false;
				std::cout << "color:" << toward << std::endl;
			}
		}
	}
	//タッチセンサーの値読み込み
	if (m_touchIn.isNew())
	{
		m_touchIn.read();
		if (changetoward == true){
			if (m_touch.data[0] == false && m_touch.data[1] == false)toward = 0;
			else if (m_touch.data[0] == true && m_touch.data[1] == false)toward = 1;
			else if (m_touch.data[0] == false && m_touch.data[1] == true)toward = 2;
			else toward = 3;
			if (toward != 0){
				changetoward = false;
				std::cout << "touch:" << toward << std::endl;

			}
		}
	}
	//超音波センサーの値読み込み
	if (m_ultrasonicIn.isNew()){
		m_ultrasonicIn.read();
		if (m_ultrasonic.ranges[0] <= u_v && changetoward == true){
			changetoward = false;
			toward = 1;
			std::cout << "ultrasonic :" << toward << std::endl;

		}
	}

	//旋回の処理
	if (changetoward == false){

		m_vel.data.vx = -m_back_vel;
		m_vel.data.vy = 0;
		m_vel.data.va = 0;
		m_velOut.write();
		std::cout << "back vx:" << m_vel.data.vx << std::endl;
		Sleep(m_back_time * 1000);

		if (toward == 1){
			m_vel.data.vx = 0;
			m_vel.data.vy = 0;
			m_vel.data.va = -m_turn_vel;
			m_velOut.write();
			std::cout << " left turn:" << m_vel.data.va << std::endl;
			Sleep(m_turn_time * 1000);
		}
		else if (toward == 2)
		{
			m_vel.data.vx = 0;
			m_vel.data.vy = 0;
			m_vel.data.va = m_turn_vel;
			m_velOut.write();
			std::cout << " right turn:" << m_vel.data.va << std::endl;
			Sleep(m_turn_time * 1000);
		}
		else{
			m_vel.data.vx = 0;
			m_vel.data.vy = 0;
			m_vel.data.va = 2 * m_turn_vel;
			m_velOut.write();
			std::cout << " back turn:" << m_vel.data.va << std::endl;
			Sleep(m_turn_time * 1000);
		}
		m_vel.data.vx = 0;
		m_vel.data.vy = 0;
		m_vel.data.va = 0;
		m_velOut.write();
		std::cout << " stop" << std::endl;
		Sleep(1000);

		m_vel.data.vx = m_front_vel;
		m_vel.data.vy = 0;
		m_vel.data.va = 0;
		changetoward = true;
		toward = 0;

		std::cout << " forward" << std::endl;

	}
	
	m_velOut.write();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ev3allsensor::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ev3allsensor::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ev3allsensor::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3allsensor::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ev3allsensor::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ev3allsensorInit(RTC::Manager* manager)
  {
    coil::Properties profile(ev3allsensor_spec);
    manager->registerFactory(profile,
                             RTC::Create<ev3allsensor>,
                             RTC::Delete<ev3allsensor>);
  }
  
};


